require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xrobocopy',
  dscmeta_resource_friendly_name: 'xRobocopy',
  dscmeta_resource_name: 'MSFT_xRobocopy',
  dscmeta_module_name: 'xRobocopy',
  dscmeta_module_version: '1.2.0.0',
  docs: 'The DSC xRobocopy resource type.
         Automatically generated from version 1.2.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_subdirectoriesincludingempty: {
      type: 'Optional[Boolean]',
      desc: 'Copy subdirectories, including Empty ones.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_additionalargs: {
      type: 'Optional[String]',
      desc: 'Robocopy has MANY configuration options.  Too many to present them all as DSC parameters effectively. Use this option to set additional parameters.  The string will be appended to the arguements list.  For a list of options run Robocopy /??? in a shell window.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_source: {
      type: 'String',
      desc: 'Source Directory, Drive or UNC path.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_destination: {
      type: 'String',
      desc: 'Destination Dir, Drive or UNC path.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_retry: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Number of Retries on failed copies: default 1 million.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_restartable: {
      type: 'Optional[Boolean]',
      desc: 'Copy files in restartable mode.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_multithreaded: {
      type: 'Optional[Boolean]',
      desc: 'Do multi-threaded copies with n threads (default 8). N must be at least 1 and not greater than 128. This option is incompatible with the /IPG and /EFSRAW options. Redirect output using /LOG option for better performance.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_appendlog: {
      type: 'Optional[Boolean]',
      desc: 'Determine whether to overwrite log file or append.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_logoutput: {
      type: 'Optional[String]',
      desc: 'Output status to LOG file.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_wait: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'Wait time between retries: default is 30 seconds.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_excludefiles: {
      type: 'Optional[String]',
      desc: 'Exclude Files matching given names/paths/wildcards.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_files: {
      type: 'Optional[String]',
      desc: 'File(s) to copy  (names/wildcards: default is all files).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
